/***************************************************************************
 * project      : Cyberjack Diagnoses Tool
    begin       : Fri Jan 26 2007
    copyright   : (C) 2007 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *             This file is licensed under the GPL version 2.              *
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ctapi_config_l.h"

#include "checksuite.h"

#include <stdio.h>
#include <string.h>
#include <errno.h>




int check() {
  CheckSuite *cs;
  std::string xmlString;
  std::string reportString;
  std::string hintString;
  bool b;
  FILE *f;

  cs=new CheckSuite();
  cs->addStandardModules();
  b=cs->performChecks(xmlString, reportString, hintString);

  f=fopen("cyberjack.xml", "w+");
  if (f) {
    fprintf(f, "%s", xmlString.c_str());
    fclose(f);
  }
  else {
    fprintf(stderr, "fopen(\"cyberjack.xml\"): %s\n", strerror(errno));
  }

  f=fopen("cyberjack-report.log", "w+");
  if (f) {
    fprintf(f, "%s", reportString.c_str());
    fclose(f);
  }
  else {
    fprintf(stderr, "fopen(\"cyberjack-report.log\"): %s\n", strerror(errno));
  }

  f=fopen("cyberjack-hints.log", "w+");
  if (f) {
    fprintf(f, "%s", hintString.c_str());
    fclose(f);
  }
  else {
    fprintf(stderr, "fopen(\"cyberjack-hints.log\"): %s\n", strerror(errno));
  }

  fprintf(stderr,
	  "\n"
	  "Es wurden 3 Dateien im aktuellen Verzeichnis angelegt:\n"
	  "- cyberjack-report.log: Enthaelt die Ergebnisse der Tests\n"
	  "- cyberjack-hints.log : Enthaelt moeglicherweise Hinweise\n"
	  "                        zu gefundenen Problemen und deren\n"
	  "                        Behebung.\n"
	  "- cyberjack.xml       : Enthaelt die Testergebnisse in fuer\n"
	  "                        den Support aufbereiteter Form.\n"
	  "Bitte senden Sie bei Problemen die Datei \"cyberjack.xml\"\n"
	  "an den Linux-Support von Reiner SCT.\n"
	  "\n");

  return 0;
}




int addFlags(unsigned int fl) {
  unsigned int oldFlags;

  if (rsct_config_init()) {
    fprintf(stderr, "ERROR: Could not init configuration.\n");
    return 2;
  }

  oldFlags=rsct_config_get_flags();
  oldFlags|=fl;
  rsct_config_set_flags(oldFlags);

  if (rsct_config_save()) {
    fprintf(stderr,
	    "ERROR: Could not save configuration, "
	    "maybe you need to be root?\n");
    rsct_config_fini();
    return 3;
  }
  rsct_config_fini();

  fprintf(stderr, "INFO: Added flags %08x (now: %08x)\n", fl, oldFlags);

  return 0;
}



int delFlags(unsigned int fl) {
  unsigned int oldFlags;

  if (rsct_config_init()) {
    fprintf(stderr, "ERROR: Could not init configuration.\n");
    return 2;
  }

  oldFlags=rsct_config_get_flags();
  oldFlags&=~fl;
  rsct_config_set_flags(oldFlags);

  if (rsct_config_save()) {
    fprintf(stderr,
	    "ERROR: Could not save configuration, "
	    "maybe you need to be root?\n");
    rsct_config_fini();
    return 3;
  }
  rsct_config_fini();

  fprintf(stderr, "INFO: Cleared flags %08x (now: %08x)\n", fl, oldFlags);

  return 0;
}





int main(int argc, char **argv) {
  if (argc>1) {
    if (strcmp(argv[1], "check")==0)
      return check();
    else if (strcmp(argv[1], "addflags")==0) {
      if (argc>2) {
	int fl;

	if (sscanf(argv[2], "%i", &fl)!=1) {
	  fprintf(stderr, "ERROR: Not an integer\n");
          return 1;
	}
	return addFlags(fl);
      }
      else {
	fprintf(stderr, "ERROR: Flags missing.\n");
        return 1;
      }
    }
    else if (strcmp(argv[1], "delflags")==0) {
      if (argc>2) {
	int fl;

	if (sscanf(argv[2], "%i", &fl)!=1) {
	  fprintf(stderr, "ERROR: Not an integer\n");
          return 1;
	}
        return delFlags(fl);
      }
      else {
	fprintf(stderr, "ERROR: Flags missing.\n");
        return 1;
      }
    }
    else {
      fprintf(stderr,
	      "Usage: %s [COMMAND] [ARGUMENTS]\n"
              "\n"
	      "COMMAND can be:\n"
	      " check:    performs system checks\n"
	      " addflags: adds flags in the CTAPI configuration file\n"
	      "           (on this system %s)\n"
              "           This command needs an argument (the flags to add)\n"
	      " delflags: clears flags in the CTAPI configuration file\n"
	      "           This command needs an argument (the flags to clear)\n"
              "\n"
	      "Examples:\n"
	      " %s check\n"
	      " %s addflags 0xffff\n"
	      " %s delflags 0xff00\n",
	      argv[0],
	      CYBERJACK_CONFIG_FILE,
	      argv[0],
	      argv[0],
	      argv[0]);
      return 1;
    }
  }
  else {
    return check();
  }
}





